var canvas;

var canvasWidth;
var canvasHeight;

var mouseX=0;
var mouseY=0;
var nFrame = 0;

var typodeObject;

var multiLinesPolygons;
var linearPolygons;
var polygons;

var transformation;
var nTransformation = 0;
var transformations;

var follow_mouseX = 0;
var follow_mouseY = 0;

var WIDTH = 850;
var FONT_SCALE = 18;
var FONT_SPACE = 4;
var INTER_LINE = 45;
var TX = 50;
var TY = 100;

//text below
var textFlowDiv = document.getElementById('flow_text');

var TEXT = "Typode is an extremely simple font described with coordinates so it can be freely reshaped, distorted or mapped. I created it to be used in certain information visualization contexts, in which text needs to adapt to specific shapes. Typode contains CAPITALS, numbers: [012345678] and symbols: [,.;:+*()<>/\\\"'=%$].\n\n>>>>>> Click to shift transformation!";

window.addEventListener('load', function() {
	setup();
	loadData();
	canvas.addEventListener("mousemove", onMouseMove, false);
	textFlowDiv.setAttribute('style', 'position:absolute;top:'+String(canvasHeight-38)+'px;left:50px;z-index:31');
});

onMouseMove = function(e){
	mouseX=window.event.clientX;
    mouseY=window.event.clientY;
}

parseData = function(data){
	
	typodeObject = JSON.parse(data);
	
	init();
}


init = function(){	
	polygons = multiLinesPolygons = polygonsFromText(TEXT, WIDTH, FONT_SCALE, FONT_SPACE, INTER_LINE, TX, TY);
	linearPolygons = polygonsFromText(TEXT, 0, FONT_SCALE, FONT_SPACE, 0, 0, 0); //used for the spiral transformation
	
	transformations = [vibrato, fishEye, wave, zoom3D, waves, spiral, rotato];
	transformation = transformations[0];
	
	//triggers the cycle
	setInterval(enterFrame, 30);
	
	
	//on mouse down the transformation is toggled
	canvas.addEventListener("mousedown", 
		function(e){
			nTransformation++;
			nTransformation = nTransformation%transformations.length;
			
			transformation = transformations[nTransformation];
			
			if(transformation==spiral || transformation==zoom3D){
				polygons = linearPolygons;
			} else {
				polygons = multiLinesPolygons;
			}
		
		}, false
	);
	
	canvas.style.cursor = 'pointer';
}


onResize=function(e){
	canvas.setAttribute('width', document.body.clientWidth);
    canvas.setAttribute('height', document.body.clientHeight); 
	
	canvasWidth=context.canvas.width  = window.innerWidth;
	canvasHeight=context.canvas.height = window.innerHeight;
	
	textFlowDiv.setAttribute('style', 'position:absolute;top:'+String(canvasHeight-38)+'px;left:50px;z-index:31');
}

enterFrame=function(){
	context.clearRect(0, 0, canvasWidth, canvasHeight);
	
	nFrame++;
	
	//follow_mouseX, follow_mouseY are used as a point that follows the cursor softly
	follow_mouseX = 0.8*follow_mouseX + 0.2*mouseX;
	follow_mouseY = 0.8*follow_mouseY + 0.2*mouseY;
	
	drawTypes();
}

drawTypes = function(){
	context.strokeStyle = 'white';
	context.lineWidth = 1;
	
	var polygon;
	var point;
	
	for(var i=0; polygons[i]!=null; i++){
		polygon = polygons[i];
		
		if(polygon.length==0) continue;
		
		context.beginPath();
		
		point = transformation(polygon[0][0], polygon[0][1]);
		
		context.moveTo(point[0], point[1]);
		
		for(var j=1; polygon[j]!=null; j++){
			point = transformation(polygon[j][0], polygon[j][1]);
			context.lineTo(point[0], point[1]);
		}
		
		context.stroke();
	}
}

////////transformation functions

identity = function(x, y){
	return [x, y];
}


fishEye = function(x, y){
	var q = 5;
	var k = 0.01;
	
	var vx = x-follow_mouseX;
	var vy = y-follow_mouseY;
	var norm = Math.sqrt(Math.pow(vx, 2)+Math.pow(vy, 2));
	var factor = (norm + (q*norm/(1+(Math.pow(k*norm,2)))))/norm;
	
	return [follow_mouseX+vx*factor, follow_mouseY+vy*factor];
}


vibrato = function(x, y){
	var r = (Math.cos(nFrame*0.035)+1)*2;
	var angle = 2*Math.PI*Math.random();
	
	return [x+r*Math.cos(angle), y+r*Math.sin(angle)];
}

rotato = function(x, y){
	var r = Math.pow((Math.cos(nFrame*0.01)+1)*0.5, 2)*INTER_LINE*0.8;
	var angle = nFrame*0.05 + x+y;
	
	return [x+r*Math.cos(angle), y+r*Math.sin(angle)];
}

wave = function(x, y){
	var r = (Math.cos(y*0.01+nFrame*0.01)+1)*200;
	var angle = 0;
	
	return [x+r*Math.cos(angle), y];
}

waves = function(x, y){
	var nl = Math.round((y-TY*0.5)/INTER_LINE);
	var yC = nl*INTER_LINE;
	
	var r = (Math.cos(nl*Math.PI+x*0.01+nFrame*0.01)+1)*0.28;
	
	return [x, yC + (y-yC)*r];
}

spiral = function(x, y){
	var a = x*0.004+follow_mouseX*0.1;
	var r = Math.pow(2, a*0.1) + 0.05*Math.pow(2, (a+2*Math.PI)*0.1)*(20-y)/20;
	
	return [canvasWidth*0.5 + r*Math.cos(a), canvasHeight*0.5 + r*Math.sin(a)];
}

zoom3D = function(x, y){
	var wF = FONT_SCALE+FONT_SPACE;
	
	var nT = Math.floor(x/wF);
	var wT = nT*wF;
	var x2 = x - wT;
	
	var z = 100*((nT-0.07*nFrame+100*TEXT.length)%TEXT.length)-299;

	var s = 5*300/(z+300);
	
	var cX = canvasWidth*0.5;
	var cY = canvasHeight*0.5;
	
	return [cX + (x2-wF*0.5+(follow_mouseX-cX)*0.2)*s, cY + (y-wF*0.5+(follow_mouseY-cY)*0.2)*s];
}


////////Typode processes

polygonsFromText = function(text, maxWidthLine, scale, space, interLines, tx, ty){
	var matrix;
	var ps = [];
	var polygon;
	var k;
	var alpha;
	
	var x0 = tx;
	var y0 = ty;
	
	var s = scale/2;
	
	for(var i=0; i<text.length; i++){
		k  = text.charAt(i);
		
		if(k=="\n" && maxWidthLine!=0){
			y0 += interLines;
			x0 = tx;
			continue;
		}
		
		matrix = typodeObject[k];
		
		alpha = matrix!=null;
		
		if(matrix==null){
			var symbolPosition = typodeObject.symbols[0].indexOf(k);
			if(symbolPosition!=-1) matrix =  typodeObject[typodeObject.symbols[1][symbolPosition]];
		}
		
		if(matrix==null) matrix = [];
		
		for(h=0; matrix[h]!=null; h++){
			polygon = [];
			for(var j= 0; matrix[h][j]!=null; j++){
				polygon[j] = [matrix[h][j][0]*s+x0, matrix[h][j][1]*s+y0];
			}
			ps.push(polygon);
		}
		
		if(!(x0==tx && k=="space")) x0+=(scale+space);
		if(maxWidthLine>0 && x0+scale+space>maxWidthLine+tx && !alpha){
			y0 += interLines;
			x0 = tx;
		}
	}
	
	return ps; 
}



////////loader

loadData = function(){
	var req = new XMLHttpRequest();
	req.onreadystatechange = onLoadComplete=function() {
		if(req.readyState==4){
			parseData(req.responseText);
		}
	};
	req.open("GET", './resources/typode.json');
	req.send("");
}


////////setup


setup = function(){
	canvas = document.getElementById('main');
	
	context = canvas.getContext('2d');

	onResize();
	
	window.addEventListener("resize", onResize, false);
}
